/*
 * @brief This header file is part of Ding RTC engine sdk for iOS/Mac system
 *
 * @copyright Copyright (c) 2024, Alibaba Inc.
 * All rights reserved
 */

#import <DingRTC/DingRtcEngineWhiteboardTypes.h>

NS_ASSUME_NONNULL_BEGIN

/** @defgroup OC_DingRtcEngineWhiteboard 白板全部接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 白板相关接口
 */

@class DingRtcEngineWhiteboard;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 白板管理类回调接口。
 */
@protocol DingRtcEngineWhiteboardManagerDelegate <NSObject>
@optional

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 * 
 * @since 3.6
 * @brief 白板开启回调通知
 * @param whiteboardId 白板Id
 */
- (void)onWhiteboardStart:(NSString *)whiteboardId config:(DingRtcWbConfig *)cfg;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 白板结束回调通知
 * @param whiteboardId 白板Id
 */
- (void)onWhiteboardStop:(NSString *)whiteboardId;

@end


/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 白板回调接口。
 */
@protocol DingRtcEngineWhiteboardDelegate <NSObject>
@optional

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 加入白板结果。
 * @param whiteboardId 白板Id
 * @param result 加入白板结果，成功返回0，失败返回错误码。
 */
- (void)whiteboard:(NSString *)whiteboardId onJoinResult:(int)result;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 离开白板结果。
 * @param whiteboardId 白板Id
 * @param result 离开白板结果，成功返回0，失败返回错误码。
 */
- (void)whiteboard:(NSString *)whiteboardId onLeaveResult:(int)result;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.5
 * @brief 白板用户更新通知。
 * @param action 0: join, 1: leave
 * @param members 变动的用户列表。
 * @param memberCount members数量。
 */
- (void)whiteboard:(NSString *)whiteboardId onUserMemberUpdate:(int)action members:(NSArray<DingRtcWbUserMember *> *)members;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板快照结果。
 * @param whiteboardId 白板Id
 * @param result 快照结果，成功返回0，失败返回错误码。
 * @param fileName 快照文件名。
 */
- (void)whiteboard:(NSString *)whiteboardId onSnapshotComplete:(int)result fileName:(NSString *)fileName;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 撤消状态。
 * @param whiteboardId 白板Id
 * @param undo 是否允许撤消。
 */
- (void)whiteboard:(NSString *)whiteboardId onUndoStatus:(BOOL)undo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 恢复状态。
 * @param whiteboardId 白板Id
 * @param redo 是否允许恢复。
 */
- (void)whiteboard:(NSString *)whiteboardId onRedoStatus:(BOOL)redo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 缩放比例变化。
 * @param whiteboardId 白板Id
 * @param scale 缩放比例。
 */
- (void)whiteboard:(NSString *)whiteboardId onZoomScaleChanged:(float)scale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 绘制事件。
 * @param whiteboardId 白板Id
 * @param event 绘制事件。
 */
- (void)whiteboard:(NSString *)whiteboardId onDrawEvent:(DingRtcWbDrawEvent)event;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 内容更新
 * @param whiteboardId 白板Id
 * @param type 内容更新类型。
 */
- (void)whiteboard:(NSString *)whiteboardId onContentUpdate:(DingRtcWbContentUpdateType)type;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 页码变化。
 * @param whiteboardId 白板Id
 * @param curPage 页码。
 * @param totalPages 总页数。
 */
- (void)whiteboard:(NSString *)whiteboardId onPageNumberChanged:(NSUInteger)curPage totalPages:(NSUInteger)totalPages;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档创建结果。
 * @param whiteboardId 白板Id
 * @param result 创建结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 */
- (void)whiteboard:(NSString *)whiteboardId onCreateDoc:(int)result docId:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档删除结果。
 * @param whiteboardId 白板Id
 * @param result 删除结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 */
- (void)whiteboard:(NSString *)whiteboardId onDeleteDoc:(int)result docId:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档切换结果。
 * @param whiteboardId 白板Id
 * @param result 切换结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 */
- (void)whiteboard:(NSString *)whiteboardId onSwitchDoc:(int)result docId:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档保存成图片结果。
 * @param whiteboardId 白板Id
 * @param result 保存结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 * @param outputDir 输出目录。
 */
- (void)whiteboard:(NSString *)whiteboardId onSaveDocComplete:(int)result docId:(NSString *)docId outputDir:(NSString *)outputDir;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档保存进度。
 * @param whiteboardId 白板Id
 * @param docId 文档ID。
 * @param current 当前进度（当前页数）。
 * @param total 总进度（总页数）。
 */
- (void)whiteboard:(NSString *)whiteboardId onSaveDocProgress:(NSString *)docId current:(NSUInteger)current total:(NSUInteger)total;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 收到自定义消息。
 * @param whiteboardId 白板Id
 * @param userId 发送方用户ID。
 * @param msg 收到的消息。
 */
- (void)whiteboard:(NSString *)whiteboardId onReceiveMessage:(NSString *)userId msg:(NSString *)msg;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 视角共享开始。
 * @param whiteboardId 白板Id
 * @param userId 用户ID。
 */
- (void)whiteboard:(NSString *)whiteboardId onVisionShareStarted:(NSString *)userId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 视角共享结束。
 * @param whiteboardId 白板Id
 * @param userId 用户ID。
 */
- (void)whiteboard:(NSString *)whiteboardId onVisionShareStopped:(NSString *)userId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档保存成缩略图结果。
 * @param whiteboardId 白板Id
 * @param result 保存结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 * @param outputDir 输出目录。
 */
- (void)whiteboard:(NSString *)whiteboardId onSaveDocThumbnailComplete:(int)result docId:(NSString *)docId outputDir:(NSString *)outputDir;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档缩略图保存进度。
 * @param whiteboardId 白板Id
 * @param docId 文档ID。
 * @param current 当前进度（当前页数）。
 * @param total 总进度（总页数）。
 */
- (void)whiteboard:(NSString *)whiteboardId onSaveDocThumbnailProgress:(NSString *)docId current:(NSUInteger)current total:(NSUInteger)total;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 图像状态变化。
 * @param whiteboardId 白板Id
 * @param url 图片URL。
 * @param state 图像状态。
 */
- (void)whiteboard:(NSString *)whiteboardId onImageStateChanged:(NSString *)url state:(DingRtcWbImageState)state;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 文件转码状态。
 * @param whiteboardId 白板Id
 * @param docId 文档ID。
 * @param transDocId 转码文档ID。
 * @param state 文件转码状态。
 */
- (void)whiteboard:(NSString *)whiteboardId onFileTranscodeState:(NSString *)docId transDocId:(NSString *)transDocId state:(DingRtcWbFileTransState)state;

@end


/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 标注回调接口。
 */
@protocol DingRtcEngineAnnotationDelegate <NSObject>
@optional

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 加入白板结果。
 * @param annotationId 标注Id
 * @param result 加入白板结果，成功返回0，失败返回错误码。
 */
- (void)annotation:(NSString *)annotationId onJoinResult:(int)result;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 离开白板结果。
 * @param annotationId 标注Id
 * @param result 离开白板结果，成功返回0，失败返回错误码。
 */
- (void)annotation:(NSString *)annotationId onLeaveResult:(int)result;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.5
 * @brief 白板用户更新通知。
 * @param action 0: join, 1: leave
 * @param members 变动的用户列表。
 * @param memberCount members数量。
 */
- (void)annotation:(NSString *)annotationId onUserMemberUpdate:(int)action members:(NSArray<DingRtcWbUserMember *> *)members;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板快照结果。
 * @param annotationId 标注Id
 * @param result 快照结果，成功返回0，失败返回错误码。
 * @param fileName 快照文件名。
 */
- (void)annotation:(NSString *)annotationId onSnapshotComplete:(int)result fileName:(NSString *)fileName;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 撤消状态。
 * @param annotationId 标注Id
 * @param undo 是否允许撤消。
 */
- (void)annotation:(NSString *)annotationId onUndoStatus:(BOOL)undo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 恢复状态。
 * @param annotationId 标注Id
 * @param redo 是否允许恢复。
 */
- (void)annotation:(NSString *)annotationId onRedoStatus:(BOOL)redo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 缩放比例变化。
 * @param annotationId 标注Id
 * @param scale 缩放比例。
 */
- (void)annotation:(NSString *)annotationId onZoomScaleChanged:(float)scale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 绘制事件。
 * @param annotationId 标注Id
 * @param event 绘制事件。
 */
- (void)annotation:(NSString *)annotationId onDrawEvent:(DingRtcWbDrawEvent)event;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 内容更新
 * @param annotationId 标注Id
 * @param type 内容更新类型。
 */
- (void)annotation:(NSString *)annotationId onContentUpdate:(DingRtcWbContentUpdateType)type;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 页码变化。
 * @param annotationId 标注Id
 * @param curPage 页码。
 * @param totalPages 总页数。
 */
- (void)annotation:(NSString *)annotationId onPageNumberChanged:(NSUInteger)curPage totalPages:(NSUInteger)totalPages;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档创建结果。
 * @param annotationId 标注Id
 * @param result 创建结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 */
- (void)annotation:(NSString *)annotationId onCreateDoc:(int)result docId:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档删除结果。
 * @param annotationId 标注Id
 * @param result 删除结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 */
- (void)annotation:(NSString *)annotationId onDeleteDoc:(int)result docId:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档切换结果。
 * @param annotationId 标注Id
 * @param result 切换结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 */
- (void)annotation:(NSString *)annotationId onSwitchDoc:(int)result docId:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档保存成图片结果。
 * @param annotationId 标注Id
 * @param result 保存结果，成功返回0，失败返回错误码。
 * @param docId 文档ID。
 * @param outputDir 输出目录。
 */
- (void)annotation:(NSString *)annotationId onSaveDocComplete:(int)result docId:(NSString *)docId outputDir:(NSString *)outputDir;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 白板文档保存进度。
 * @param annotationId 标注Id
 * @param docId 文档ID。
 * @param current 当前进度（当前页数）。
 * @param total 总进度（总页数）。
 */
- (void)annotation:(NSString *)annotationId onSaveDocProgress:(NSString *)docId current:(NSUInteger)current total:(NSUInteger)total;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 收到自定义消息。
 * @param annotationId 标注Id
 * @param userId 发送方用户ID。
 * @param msg 收到的消息。
 */
- (void)annotation:(NSString *)annotationId onReceiveMessage:(NSString *)userId msg:(NSString *)msg;

@end


/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 白板管理器。
 */
__attribute__((visibility("default"))) @interface DingRtcEngineWhiteboardManager : NSObject

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 * 
 * @since 3.6
 * @brief 设置白板管理器监听器。
 * @param delegate  用于接收来自白板管理器的事件
 */
- (void)setDelegate:(id<DingRtcEngineWhiteboardManagerDelegate>_Nullable)delegate;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 * 
 * @since 3.6
 * @brief 设置白板缓存路径。
 * @param path 白板缓存路径
 * @return
 * - 0：成功；
 * - 非0：失败。
 * @note 默认为tmp目录
 */
- (int)setCachePath:(NSString *)path;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6.0
 * @brief 创建白板对象
 * @param whiteboardId 白板Id（ding开头的为内部使用字段）
 * @param config 白板配置参数
 * @return 白板对象
 */
- (DingRtcEngineWhiteboard *)createWhiteboard:(NSString *)whiteboardId withConfig:(DingRtcWbConfig *)config;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6.0
 * @brief 获取白板对象
 * @param whiteboardId 白板Id（ding开头的为内部使用字段）
 * @return 白板对象
 */
- (DingRtcEngineWhiteboard *)getWhiteboard:(NSString *)whiteboardId;

@end


/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 白板基类。
 */
__attribute__((visibility("default"))) @interface DingRtcEngineWhiteboardBase : NSObject

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 打开白板。
 * @param view 视图。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)open:(DingRtcOSView *)view;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 关闭白板。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)close;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 加入白板。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)join;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 离开白板。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)leave;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 结束白板。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)stop;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置白板角色。
 * @param role 白板角色。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setRole:(DingRtcWbRoleType)role;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置工具类型。
 * @param toolType 工具类型。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setToolType:(DingRtcWbTool)toolType;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置线宽。
 * @param size 线宽。有效值为1-20。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setLineWidth:(int)size;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置颜色。
 * @param color 颜色。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setColor:(DingRtcWbColor *)color;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置填充颜色。
 * @param color 填充颜色。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setFillColor:(DingRtcWbColor *)color;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置填充类型。
 * @param type 填充类型。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setFillType:(DingRtcWbFillType)type;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置字体样式。
 * @param style 字体样式。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setFontStyle:(DingRtcWbFontStyle)style;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置字体大小。
 * @param size 字体大小。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setFontSize:(int)size;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 撤消。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)undo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 重做。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)redo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard

 * @since 3.6
 * @brief 获取工具类型。
 * @return WBToolType 工具类型。
*/
- (DingRtcWbTool)getToolType;

/**
 * @ingroup OC_DingRtcEngineWhiteboard

 * @since 3.6
 * @brief 是否可以撤消。
 * @return
 * - YES：可以；
 * - NO：不可以。
*/
- (BOOL)getUndoStatus;

/**
 * @ingroup OC_DingRtcEngineWhiteboard

 * @since 3.6
 * @brief 是否可以恢复重做。
 * @return
 * - YES：可以；
 * - NO：不可以。
*/
- (BOOL)getRedoStatus;

/**
 * @ingroup OC_DingRtcEngineWhiteboard

 * @since 3.6
 * @brief 添加图章资源。
 * @param stamp 图章对象。
 * @return
 * - 0：成功；
 * - 非0：失败。
*/
- (int)addStamp:(DingRtcWbStamp *)stamp;

/**
 * @ingroup OC_DingRtcEngineWhiteboard

 * @since 3.6
 * @brief 设置图章资源。
 * @param stampId 图章资源ID。
 * @return
 * - 0：成功；
 * - 非0：失败。
*/
- (int)setStamp:(NSString *)stampId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置缩放比例。
 * @param scale 缩放比例。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setZoomScale:(float)scale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置缩放比例及基于原始中心点缩放后的平移。
 * @param scale 缩放比例。
 * @param tx 缩放后x轴平移。
 * @param ty 缩放后y轴平移。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setZoomScaleWithTranslate:(float)scale tx:(float)tx ty:(float)ty;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置最小缩放比例。
 * @param scale 缩放比例。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setMinZoomScale:(float)scale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置最大缩放比例。
 * @param scale 缩放比例。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setMaxZoomScale:(float)scale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 获取缩放比例。
 * @return 缩放比例。
 */
- (float)getZoomScale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 获取最小缩放比例。
 * @return 缩放比例。
 */
- (float)getMinZoomScale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 获取最大缩放比例。
 * @return 缩放比例。
 */
- (float)getMaxZoomScale;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置缩放模式。
 * @param mode 缩放模式。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setScalingMode:(DingRtcWbScalingMode)mode;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 获取当前白板页码。
 * @return 当前白板页码。
 */
- (NSUInteger)getCurrentPageNumber;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 获取白板总页数。
 * @return 白板总页数。
 */
- (NSUInteger)getTotalNumberOfPages;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 添加一页到最后。
 * @param autoSwitch 自动切换到新添加的页面。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)addPage:(BOOL)autoSwitch;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 插入一页到指定页之后。
 * @param pageNo 指定页的页码。
 * @param autoSwitch 自动切换到新添加的页面。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)insertPage:(NSUInteger)pageNo autoSwitch:(BOOL)autoSwitch;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 删除一页。
 * @param pageNo 被删除页页码。
 * @param switchNext 切换到删除页的下一页。
 * @note 删除页的默认行为是切换到删除页的前一页。设置 switchNext 为 YES 改为切换到删除页的下一页。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)removePage:(NSUInteger)pageNo switchNext:(BOOL)switchNext;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 跳转到指定页。
 * @param pageNo 跳转页页码。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)gotoPage:(NSUInteger)pageNo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 下一页。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)nextPage;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 上一页。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)prevPage;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 获取当前白板文档Id。
 * @return 当前白板文档Id。
 */
- (NSString *)getCurrentDocId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 获取白板文档信息。
 * @param docId 白板文档Id。
 * @return 白板文档信息。
 */
- (DingRtcWbDocInfo *)getDocInfo:(NSString*)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.5
 * @brief 获取白板文档列表。
 * @return 白板文档ID列表
 */
- (NSArray<NSString *> *)getDocList;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 * @since 3.5
 * @brief 基于内容创建白板文档。
 * @param contents 文档信息。
 * @param autoSwitch 自动切换到新创建的文档。
 * @return const char* 白板文档ID。
 * @note contents.transDocId不为空，则查询转码文件信息，忽略contents.urls参数
 */
- (NSString *)createDoc:(DingRtcWbDocContents *)contents autoSwitch:(BOOL)autoSwitch;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 基于外部内容创建白板文档。
 * @param contents 外部文档信息。
 * @param autoSwitch 自动切换到新创建的文档。
 * @return 白板文档Id。
 */
- (NSString *)createDocWithExternal:(DingRtcWbDocExtContents *)contents autoSwitch:(BOOL)autoSwitch;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 删除白板文档。
 * @param docId 白板文档Id。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)deleteDoc:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 切换白板文档。
 * @param docId 白板文档ID。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)switchDoc:(NSString *)docId;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 复制白板文档页内容。
 * @param srcDocId 源白板文档ID。
 * @param srcPageNum 源白板文档页码。
 * @param dstDocId 目标白板文档ID。
 * @param dstPageNum 目标白板文档页码。
 * @param clearDstPage 是否清空目标白板文档页。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)copyDocPage:(NSString *)srcDocId srcPageNum:(NSUInteger)srcPageNum
          dstDocId:(NSString *)dstDocId dstPageNum:(NSUInteger)dstPageNum
      clearDstPage:(BOOL)clearDstPage;
/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 保存白板文档为多张图片。
 * @param docId 白板文档Id。
 * @param outputDir 输出目录
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)saveDocToImages:(NSString *)docId outputDir:(NSString *)outputDir;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 发送自定义消息给指定用户。
 * @param userId 用户Id。
 * @param msg 要发送的消息，最大不超过16KB。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)sendUserMessage:(NSString *)userId msg:(NSString *)msg;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 广播消息给所有用户。
 * @param msg 要发送的消息，最大不超过16KB。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)broadcastMessage:(NSString *)msg;

@end


/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 白板类。
 */
__attribute__((visibility("default"))) @interface DingRtcEngineWhiteboard : DingRtcEngineWhiteboardBase

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置白板监听器。
 * @param delegate  用于接收来自白板的事件
 */
- (void)setDelegate:(id<DingRtcEngineWhiteboardDelegate>_Nullable)delegate;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 设置白板参数。
 * @param option 参数类别。
 * @param param 参数，具体参数需要遵循 option 定义的参数结构。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setOption:(DingRtcWbOptionType)option param:(id)param;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 清除白板内容。
 * @param param 清除参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)clearContents:(DingRtcWbClearParam *)param;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 开始视角共享。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)startShareVision;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 停止视角共享。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)stopShareVision;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 开始视角跟随。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)startFollowVision;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 停止视角跟随。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)stopFollowVision;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 快照。
 * @param mode 快照模式
 * @param outputDir 输出目录。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)snapshot:(DingRtcWbSnapshotMode)mode outputDir:(NSString*)outputDir;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 设置背景色。
 * @param color 背景色。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setBackgroundColor:(DingRtcWbColor *)color;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 设置背景图片
 * @param imageUrl 图片地址，可以为本地路径或网络路径
 * @param pageNo 当前文档页码
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setBackgroundImage:(NSString *)imageUrl pageNo:(NSUInteger)pageNo;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 添加图片
 * @param imageUrl 图片地址，可以为本地路径或网络路径
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)addImageFile:(NSString *)imageUrl;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 保存白板文档为多张缩略图。
 * @param docId 白板文档Id
 * @param outputDir 输出目录。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)saveDocToThumbnails:(NSString *)docId outputDir:(NSString*)outputDir;

@end


/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 标注类。
 */
__attribute__((visibility("default"))) @interface DingRtcEngineAnnotation : DingRtcEngineWhiteboardBase

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置标注监听器。
 * @param delegate  用于接收来自标注的事件
 */
- (void)setDelegate:(id<DingRtcEngineAnnotationDelegate>_Nullable)delegate;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 设置标注参数。
 * @param option 参数类别。
 * @param param 参数，具体参数需要遵循 option 定义的参数结构。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setOption:(DingRtcAnnoOptionType)option param:(id)param;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 清除标注内容。
 * @param param 清除参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)clearContents:(DingRtcWbClearParam *)param;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 *
 * @since 3.6
 * @brief 快照。
 * @param outputDir 输出目录。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)snapshot:(NSString*)outputDir;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置标注区域大小。
 * @param width 宽度。
 * @param height 高度。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setAspectSize:(int)width height:(int)height;

/**
 * @ingroup OC_DingRtcEngineWhiteboard
 
 * @since 3.6
 * @brief 设置标注窗口位置及大小。
 * @param rect 窗口位置及大小。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setWindowPos:(CGRect)rect;

@end

NS_ASSUME_NONNULL_END
